@@ -1,22 +1,23 @@
-__version__ = '2.12.5'
+__version__ = '2.19.3'
 import importlib
 
+
 __min_reqs__ = [
             'pip>=23.0',
             'typer>=0.6.1',
-            'numpy>=1.23.4',
-            'jax>=0.4.4',
-            'jaxlib>=0.4.4',
+            'numpy>=1.21.0',
+            'jax>=0.4.20',
+            'jaxlib>=0.4.20',
             'matplotlib>=3.5.1',
             'pandas>=1.4.1',
-            'scipy>=1.10.1',
+            'scipy>=1.11.3',
             'statsmodels>=0.13.2',
-            'betanegbinfit>=1.8.1',
-            'datatable>=1.0.0',
-            'dill>=0.3.6',
+            'betanegbinfit>=1.9.9',
+            'datatable>=1.0.0' ,
+            'dill>=0.3.7',
             'rich>=12.6.0',
             'portion>=2.3.0',
-            'pysam>=0.19.1'
+            'pysam>=0.22.0'
            ]
 
 def versiontuple(v):
@@ -24,11 +25,11 @@ def versiontuple(v):
 
 def check_packages():
     for req in __min_reqs__:
-        module, ver = req.split('>=')
-        ver = versiontuple(ver)
         try:
+            module, ver = req.split(' @').split('>=')
+            ver = versiontuple(ver)
             v = versiontuple(importlib.import_module(module).__version__)
-        except AttributeError:
+        except (AttributeError, ValueError):
             continue
         if v < ver:
             raise ImportError(f'Version of the {module} package should be at least {ver} (found: {v}).')
