@@ -4,8 +4,9 @@ import numpy as np
 import dill
 import os
 import re
-from .utils import get_init_file, openers, parse_filenames
+from .utils import get_init_file, openers, select_filenames
 from multiprocessing import cpu_count, Pool, Manager
+from collections import defaultdict
 from functools import partial
 from itertools import islice
 from statsmodels.stats import multitest
@@ -30,7 +31,7 @@ def combine_p_values_logit(pvalues):
         pval = st.distributions.chi2.sf(statistic, 2 * k)
     return pval
 
-def combine_es(es, pvalues):
+def combine_es(es, pvalues, uniform_weights=False):
     pvalues = np.array(list(map(float, pvalues)))
     es = np.array(es)
     weights = -np.log10(pvalues)
@@ -38,6 +39,8 @@ def combine_es(es, pvalues):
     if not inds.sum():
         return np.mean(es)
     weights = weights[inds]
+    if uniform_weights:
+        weights[:] = 1.0
     es = es[inds]
     s = weights.sum()
     if s == 0:
@@ -46,7 +49,28 @@ def combine_es(es, pvalues):
         weights /= s
     return np.sum(weights * es)
 
-def combine_stats(inds, snvs, stats, groups, min_cnt_sum=20):
+def estimate_min_coverage(test, es_cutoff: float = 1, pval_cutoff: float = 0.5):
+    res = defaultdict(dict)
+    for allele in test:
+        for bad in test[allele]:
+            it = test[allele][bad]
+            counts = np.array(list(it.keys()))
+            coverage = counts.sum(axis=1)
+            stop = False
+            for cov in np.unique(coverage):
+                ind = coverage == cov
+                pairs = counts[ind]
+                for t in pairs:
+                    pvalue, es = it[tuple(t)]
+                    if (es > es_cutoff) and (pvalue < pval_cutoff):
+                        stop = True
+                        break
+                if stop:
+                    break
+            res[allele][bad] = cov if stop else float('inf')
+    return res
+
+def combine_stats(inds, snvs, stats, groups, min_cnt_sum=20, uniform_weights=False):
     pvalues = list()
     es = list()
     ks = list()
@@ -56,7 +80,7 @@ def combine_stats(inds, snvs, stats, groups, min_cnt_sum=20):
         if groups:
             lt = filter(lambda x: x[0] in groups, lt)
         lt = [t[1:] for t in lt]
-        if not lt or max(sum(t[:-1]) for t in lt) < min_cnt_sum:
+        if not lt or not max(sum(t[:-1]) >= min_cnt_sum[t[-1]] for t in lt):
             ref = alt = ref_es = alt_es = np.nan
             k = None
         else:
@@ -64,8 +88,8 @@ def combine_stats(inds, snvs, stats, groups, min_cnt_sum=20):
             alt_pvals, alt_es = zip(*[stats['alt'][t[-1]][t[:-1]] for t in lt])
             ref = combine_p_values_logit(ref_pvals)
             alt = combine_p_values_logit(alt_pvals)
-            ref_es = combine_es(ref_es, ref_pvals)
-            alt_es = combine_es(alt_es, alt_pvals)
+            ref_es = combine_es(ref_es, ref_pvals, uniform_weights=uniform_weights)
+            alt_es = combine_es(alt_es, alt_pvals, uniform_weights=uniform_weights)
         pvalues.append((ref, alt))
         es.append((ref_es, alt_es))
         ks.append(k)
@@ -76,11 +100,8 @@ def batched(iterable, n):
     while (batch := tuple(islice(it, n))):
         yield batch
 
-def combine(name: str, group_files=None, alpha=0.05, min_cnt_sum=20, filter_id=None, filter_chr=None, subname=None, n_jobs=1, save_to_file=True):
-    if group_files is None:
-        group_files = list()
-    else:
-        group_files = parse_filenames(group_files)
+def combine(name: str, group_files=None, alpha=0.05, min_cnt_sum=20, adaptive_min_cover=False, adaptive_es=1.0, adaptive_pval=0.05,
+            uniform_weights=False, filter_id=None, filter_chr=None, subname=None, n_jobs=1, save_to_file=True):
     if filter_chr is not None:
         filter_chr = re.compile(filter_chr)
     if filter_id is not None:
@@ -94,6 +115,12 @@ def combine(name: str, group_files=None, alpha=0.05, min_cnt_sum=20, filter_id=N
         snvs = init['snvs']
         scorefiles = init['scorefiles']
         del init
+    if group_files is None:
+        group_files = list()
+    else:
+        group_files = select_filenames(group_files, scorefiles)
+        if not group_files:
+            raise SyntaxError('No files found for given pattern(s).')
     filename = f'{name}.test.{compressor}'
     with open(filename, 'rb') as f:
         stats = dill.load(f)
@@ -104,6 +131,12 @@ def combine(name: str, group_files=None, alpha=0.05, min_cnt_sum=20, filter_id=N
             groups.add(i)
         except ValueError:
             logging.error(f'Unknown scorefile {file}')
+    if adaptive_min_cover:
+        adaptive_coverage = estimate_min_coverage(stats, es_cutoff=adaptive_es, pval_cutoff=adaptive_pval)
+        min_coverage = {bad: min(adaptive_coverage['ref'][bad], adaptive_coverage['alt'][bad]) for bad in adaptive_coverage['ref']}
+    else:
+        adaptive_coverage = None
+        min_coverage = {bad: min_cnt_sum for bad in stats['ref']}
     del scorefiles
     ref_comb_pvals = list()
     alt_comb_pvals = list()
@@ -117,12 +150,13 @@ def combine(name: str, group_files=None, alpha=0.05, min_cnt_sum=20, filter_id=N
         its = list(filter(lambda x: x[1][0][1] and filter_id.match(x[1][0][1]), its))
     if filter_chr:
         its = list(filter(lambda x: filter_chr.match(x[0]), its))
-
+    
     with Manager() as manager:
         if n_jobs > 1:
             its = manager.list(its)
         with Pool(n_jobs) as p:
-            f = partial(combine_stats, snvs=its, stats=stats, groups=groups, min_cnt_sum=min_cnt_sum)
+            f = partial(combine_stats, snvs=its, stats=stats, groups=groups, min_cnt_sum=min_coverage,
+                        uniform_weights=uniform_weights)
             if n_jobs > 1:
                 sz = int(np.ceil(len(its) / n_jobs))
                 inds = batched(range(len(its)), sz)
@@ -165,4 +199,4 @@ def combine(name: str, group_files=None, alpha=0.05, min_cnt_sum=20, filter_id=N
     if save_to_file:
         with open(f'{name}.comb.{compressor}', 'wb') as f:
             dill.dump(res, f)
-    return res
+    return res, adaptive_coverage
\ No newline at end of file
