@@ -70,7 +70,7 @@ def _read_bedlike(filename: str):
     except StopIteration:
         bad_col = len(names)
         names.append('bad')
-        df['db'] = 0
+        df['bad'] = 0
     met.add(db_col)
     inds = (chr_col, start_col, id_col, ref_count_col, alt_count_col,  ref_col, alt_col, qual_col, db_col, bad_col)
     res = list()
@@ -218,7 +218,8 @@ def file_to_table(filename: str, counts=None, min_qual=10, min_cnt=5,
                         lt.append((name, row.ref, ','.join(row.alts)))
                     else:
                         if snp_bad_check and lt[1][-1] != bad:
-                            raise Exception(f'SNV at {chrom}:{start} comes from at least two different BADs: {bad} and {lt[1][-1]}.')
+                            raise Exception(f'SNV at {chrom}:{start} comes from at least two different BADs: {bad} and {lt[1][-1]}.'
+                                            ' If that is expected, consider setting --no-snp-bad-check flag.')
                     lt.append((filename_id, ref, alt, bad))
         if flag and snps_set is not None:
             snps_set[bad].add((chrom, start))
