@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-from .utils import get_init_file, openers
+from .utils import get_init_file, openers, scorefiles_qc
 from collections import defaultdict
 import pandas as pd
 import numpy as np
@@ -26,7 +26,25 @@ def export_counts(project, out: str, bad: float = None):
     else:
         df = pd.DataFrame(counts_d[bad], columns=['Ref', 'Alt', 'N'])
         df.to_csv(out, sep='\t', index=None)
-    
+
+def export_scorefiles_qc(project, out: str):
+    covers, biases = scorefiles_qc(project)
+    bads = sorted(project['counts']) + [None]
+    scorefiles = project['scorefiles']
+    for bad in bads:
+        subfolder = os.path.join(out, f'BAD{bad:.2f}') if bad else out
+        os.makedirs(subfolder, exist_ok=True)
+        label = list()
+        cover = list()
+        bias = list()
+        for f in covers:
+            c = covers[f][bad]
+            if c:
+                cover.append(c)
+                bias.append(biases[f][bad])
+                label.append(scorefiles[f])
+        df = pd.DataFrame([label, bias, cover], index=['name', 'bias', 'cover']).T
+        df.to_csv(os.path.join(subfolder, 'scorefiles_qc.tsv'), sep='\t', index=None)
 
 def _export_params(fit, out: str, allele: str, bad: float):
     df = pd.DataFrame(fit[allele][bad]['params'])
@@ -208,12 +226,14 @@ def export_combined_pvalues(project, out: str, sample_info=False, subname=None):
             bads.append(bad)
         mean_bad = sum(bads) / len(bads)
         bads = ','.join(map(str, bads))
+        max_cover = (np.array(list(map(int, ref_counts))) + np.array(list(map(int, alt_counts)))).max()
         ref_counts = ','.join(ref_counts);alt_counts = ','.join(alt_counts)
         ref_pvals = ','.join(ref_pvals); alt_pvals = ','.join(alt_pvals)
         ref_eses = ','.join(ref_eses); alt_eses = ','.join(alt_eses)
         n = len(scores_f)
         scores_f = ','.join(scores_f)
         d['#chr'].append(chr); d['start'].append(pos); d['end'].append(end); d['mean_bad'].append(mean_bad); d['id'].append(name)
+        d['max_cover'].append(max_cover)
         d['ref'].append(ref); d['alt'].append(alt); d['n_reps'].append(n);
         
         if sample_info:
@@ -236,6 +256,52 @@ def export_combined_pvalues(project, out: str, sample_info=False, subname=None):
         os.makedirs(folder, exist_ok=True)
     pd.DataFrame(d).to_csv(out, sep='\t', index=None)
 
+def export_anova(project, out: str, subname=None):
+    if type(project) is str:
+        file = get_init_file(project)
+        compression = file.split('.')[-1]
+        open = openers[compression]
+        with open(file, 'rb') as snvs,  open(f'{project}.anova.{compression}', 'rb') as diff:
+            diff = dill.load(diff)[subname]
+    else:
+        snvs, diff = project
+        diff = diff[subname]
+    tests = diff['tests']
+    snvs = diff['snvs']
+    chrom = list(); start = list(); end = list(); name = list(); bad = list(); ref = list(); alt = list()
+    a_ref_counts = list(); b_ref_counts = list()
+    a_alt_counts = list(); b_alt_counts = list()
+    es_count = list()
+    # ref_es_count = list(); alt_es_count = list()
+    for ind in tests['ind']:
+        for s in snvs:
+            try:
+                t = s[ind]
+                if t:
+                    break
+            except KeyError:
+                continue
+        n, r, a = t[0]
+        chrom.append(ind[0]); start.append(ind[1]); end.append(start[-1] + 1); name.append(n)
+        ref.append(r); alt.append(a)
+        bad.append(t[-1][-1])
+    diff = tests.drop('ind', axis=1)
+   
+    df = pd.DataFrame({'#chr': chrom, 'start': start, 'end': end, 'mean_bad': bad, 'id': name, 'ref': ref, 'alt': alt })
+    diff = pd.concat([df, diff], axis=1)
+    t = diff['ref_pval'] < diff['alt_pval']
+    diff['scoring_model'] = ['ref|alt' if v else 'alt|ref' for v in t]
+    diff['pval'] = None
+    diff.loc[t, 'pval'] = diff.loc[t, 'ref_pval']
+    diff.loc[~t, 'pval'] = diff.loc[~t, 'alt_pval']
+    diff['fdr_pval'] = None
+    diff.loc[t, 'fdr_pval'] = diff.loc[t, 'ref_fdr_pval']
+    diff.loc[~t, 'fdr_pval'] = diff.loc[~t, 'alt_fdr_pval']
+    folder, _ = os.path.split(out)
+    if folder:
+        os.makedirs(folder, exist_ok=True)
+    diff.to_csv(out, sep='\t', index=None)
+
 def export_difftests(project, out: str,  sample_info=False, subname=None):
     if type(project) is str:
         file = get_init_file(project)
@@ -327,6 +393,7 @@ def export_all(name: str, out: str, sample_info: bool = None):
     with open(file, 'rb') as init:
         init = dill.load(init)
     export_counts(init, out)
+    export_scorefiles_qc(init, out)
     try:
         with open(f'{name}.fit.{compression}', 'rb') as f:
             fit = dill.load(f)
@@ -344,6 +411,15 @@ def export_all(name: str, out: str, sample_info: bool = None):
                                  sample_info=sample_info)
     except FileNotFoundError:
         pass     
+    try:
+        with open(f'{name}.anova.{compression}', 'rb') as f:
+            anova = dill.load(f)
+            subfolder = os.path.join(out, 'anova')
+            t = (init, anova)
+            for subname in anova:
+                export_anova(t, os.path.join(subfolder, f'{subname}.tsv' if subname else 'anova.tsv'), subname=subname)
+    except FileNotFoundError:
+        pass   
             
     try:
         with open(f'{name}.test.{compression}', 'rb') as f:
@@ -363,6 +439,6 @@ def export_all(name: str, out: str, sample_info: bool = None):
 
 def export_demo(path: str = str()):
     folder = os.path.join(os.path.split(os.path.realpath(__file__))[0], 'data')
-    filename = os.path.join(folder, 'chip.tar.gz')
+    filename = os.path.join(folder, 'dnase_k562.tar.gz')
     with tarfile.open(filename, 'r:gz') as f:
         f.extractall(os.path.join(path, 'scorefiles'))
