@@ -11,7 +11,7 @@ from jax import __version__ as jax_version
 from scipy import __version__ as scipy_version
 from rich.progress import Progress, SpinnerColumn, TextColumn
 from rich.table import Table
-from .diff import differential_test
+from .diff import differential_test, anova_test
 from .create import create_project
 from .combine import combine
 from .tests import test, binom_test
@@ -20,11 +20,15 @@ from .fit import fit
 from time import time
 from dill import __version__ as dill_version
 import importlib
+import logging
 from . import export
 from . import __version__ as mixalime_version
 import json
 from . import plot
 
+logging.getLogger("jax._src.xla_bridge").addFilter(logging.Filter("No GPU/TPU found, falling back to CPU."))
+logging.getLogger("jax._src.xla_bridge").addFilter(logging.Filter("An NVIDIA GPU may be present on this machine, but a CUDA-enabled jaxlib is not installed. Falling back to cpu."))
+
 __all__ = ['main']
 
 class Compression(str, Enum):
@@ -75,7 +79,9 @@ class SmallDatasetStrategy(str, Enum):
     conservative = 'conservative'
     fixed_r = 'fixed_r'
    
-
+class ESMode(str, Enum):
+    entropy = 'entropy'
+    mean = 'mean'
 
 
 class OrderCommands(TyperGroup):
@@ -239,7 +245,8 @@ BED-like file formats):
 A [i]scorefiles[/i] folder should appear now in a working directory with plenty of BED-like files.\
 First, we'd like to parse those files into a [bold]MixALime[/bold]-friendly and efficient data structures for further usage, as well as perform some \
 basic filtering if necessary:
-[magenta]>[/magenta] [cyan]mixalime create myprojectname [i]scorefiles[/i][/cyan]
+[magenta]>[/magenta] [cyan]mixalime create myprojectname [i]scorefiles[/i] --no-snp-bad-check[/cyan]
+Here, --no-snp-bad-check is necessarily for our demo: we used cancer data from the K562 cell line, and SNVs can indeed come from different BAD regions.
 Then we fit model parameters to the data with Negative Binomial distribution:
 [magenta]>[/magenta] [cyan]mixalime fit myprojectname NB[/cyan]
 Next we obtain raw p-values:
@@ -267,8 +274,9 @@ help_str = 'Initialize [bold]MixALime[/bold] projects initial files: do parsing
 def _create(name: str = Argument(..., help='Project name. [bold]MixALime[/bold] will produce files for internal usage that start with [cyan]'
                                             'name[/cyan].'),
             files: List[Path] = Argument(..., help='A list (if applicable, separated by space) of either filenames in VCF or BED-like format, '
-                                                    'paths to folder with those files or paths to files that contain list of paths to those '
-                                                    'files. '),
+                                                    'paths to folder with those files, paths to files that contain list of paths to those '
+                                                    'files or  mask (masks should start with "[yellow]m:[/yellow]"'
+                                                    'prefix, e.g. "m:vcfs/*_M_*.vcf.gz")  '),
             bad_maps: Path = Option(None, help='A path to an interval file that separates genome into BADs. The file must be tabular with '
                                               'columns "[bold]chr[/bold]", "[bold]start[/bold]", "[bold]end[/bold]", "[bold]bad[/bold]".'
                                               ' The "[bold]end[/bold]" column is optional, if it is not present, then the file is '
@@ -292,6 +300,8 @@ def _create(name: str = Argument(..., help='Project name. [bold]MixALime[/bold]
             pretty: bool = Option(True, help='Use "rich" package to produce eye-candy output.')):
     if max_cover is None:
         max_cover = float('inf')
+    if type(compression) is Compression:
+        compression = str(compression.value)
     files = list(map(str, files))
     if bad_maps:
         bad_maps = str(bad_maps)
@@ -318,15 +328,15 @@ def _create(name: str = Argument(..., help='Project name. [bold]MixALime[/bold]
             print('Warning: BAD=0 is present in the data. Please remove BAD=0 occurrences or substitute them with BAD=1.')
     rows = rows
     if pretty:
-        table = Table('BAD', 'SNVs', 'Samples/reps')
+        table = Table('BAD', 'SNVs', 'Obvservations')
         for row in rows:
             table.add_row(*row)
         rprint(table)
-        rprint(f'Total SNVs: {tot_snv}, total samples/reps: {tot_samples}')
+        rprint(f'Total unique SNVs: {tot_snv}, total observations: {tot_samples}')
     else:
-        print('BAD\tSNVs\tSamples/reps')
+        print('BAD\tSNVs\tObvservations')
         print('\n'.join(['\t'.join(row) for row in rows]))     
-        print(f'Total SNVs: {tot_snv}, total samples/reps: {tot_samples}')    
+        print(f'Total unique SNVs: {tot_snv}, total observations: {tot_samples}')    
     update_history(name, 'create', files=files, bad_maps=bad_maps, default_bad=default_bad, drop_bad=drop_bad, min_qual=min_qual,
                    min_cnt=min_cnt, max_cover=max_cover, filter_db=filter_db, filter_rs=filter_rs, symmetrify=symmetrify,
                    filter_name=filter_name, filter_chr=filter_chr, compression=compression, snp_bad_check=snp_bad_check,
@@ -341,8 +351,7 @@ def _create(name: str = Argument(..., help='Project name. [bold]MixALime[/bold]
 @app.command('fit')
 def _fit(name: str = Argument(..., help='Project name.'),
          dist: Dist = Argument(..., help='Name of distribution that will be used in the mixture model.'),
-         model: Model = Option(Model.window.value, help='Model to be used.'),
-         left: int = Option(4, help='Left-truncation bound. If None, then it will be estimated from the counts data as the minimal present count minus 1.'),
+         left: int = Option(None, help='Left-truncation bound. If None, then it will be estimated from the counts data as the minimal present count minus 1.'),
          estimate_p: bool = Option(False, help='If True, then p will be estimated instead of assuming it to be fixed to bad / (bad + 1)'),
          window_size: int = Option(10000, help='Has effect only if [cyan]model[/cyan] = "[bold]window[/bold]", sets the required minimal window size.'),
          window_behavior: WindowBehavior = Option(WindowBehavior.both.value, help='If "[bold]both[/bold]", then window is expanded in 2 directions. '
@@ -355,17 +364,19 @@ def _fit(name: str = Argument(..., help='Project name.'),
                                               ' number of samples.'),
          adjust_line: bool = Option(False, help='Line parameter beta and mu will be reestimated without a loss of likelihood so they differ'
                                                 ' as little as possible from the previous b and mu estimates.'),
-         k_left_bound: int = Option(1, help='Minimal allowed value for concentration parameter [italic]k[/italic].'),
+         k_left_bound: int = Option(0, help='Minimal allowed value for concentration parameter [italic]k[/italic].'),
          max_count: int = Option(None, help='Maximal number of counts for an allele (be it ref or alt).'),
-         max_cover: int = Option(None, help='Maximal sum of ref + alt.'), 
-         regul_alpha: float = Option(0.0, help='Regularization/prior strength hyperparameter alpha. Valid only for [cyan]model[/cyan]='
-                                               '[yellow]window[/yellow].'),
-         regul_n: bool = Option(True, help='Multiply alpha by a number of SNV records captured by a particular window. Valid only for '
-                                           '[cyan]model[/cyan]=[yellow]window[/yellow].'),
-         regul_slice: bool = Option(True, help='Multiply alpha by a an average slice captured by a particular window. Valid only for '
-                                           '[cyan]model[/cyan]=[yellow]window[/yellow].'),
-         regul_prior: Prior = Option('laplace', help='Prior distribution used to penalize concentration parameter kappa. Valid only for '
-                                                     '[cyan]model[/cyan]=[yellow]window[/yellow].'),
+         max_cover: int = Option(None, help='Maximal sum of ref + alt.'),
+         regul: bool = Option(False, help='Apply regularization/encourage high kappa parameter estimates. Valid only for [cyan]dist[/cyan]='
+                                               '[yellow]BetaNB[/yellow].'),
+         regul_alpha: float = Option(1.0, help='Regularization/prior strength hyperparameter alpha. Valid only for [cyan]dist[/cyan]='
+                                               '[yellow]BetaNB[/yellow].'),
+         regul_n: bool = Option(True, help='Multiply alpha by a number of SNV records captured by a particular window. Valid only for [cyan]dist[/cyan]='
+                                               '[yellow]BetaNB[/yellow].'),
+         regul_slice: bool = Option(True, help='Multiply alpha by a an average slice captured by a particular window. Valid only for [cyan]dist[/cyan]='
+                                               '[yellow]BetaNB[/yellow].'),
+         regul_prior: Prior = Option('laplace', help='Prior distribution used to penalize concentration parameter kappa. Valid only for [cyan]dist[/cyan]='
+                                               '[yellow]BetaNB[/yellow].'),
          std : bool = Option(False, help='Compute standard errors for parameter estimates. Note that it may significantly increase computation'
                                          ' time.'),
          fix_params : str = Option(None, help='Parameters that are not estimated, but fixed to a constant float argument instead. This argument '
@@ -391,6 +402,7 @@ def _fit(name: str = Argument(..., help='Project name.'),
     
     start_est = True
     apply_weights = False
+    model = Model.window.value
     if max_count is None:
         max_count = float('inf')
     if max_cover is None:
@@ -419,16 +431,16 @@ def _fit(name: str = Argument(..., help='Project name.'),
     fit(name, dist=dist, model=model, left=left, estimate_p=estimate_p, window_size=window_size, 
         window_behavior=window_behavior, min_slices=min_slices, adjust_line=adjust_line, k_left_bound=k_left_bound,
         max_count=max_count, max_cover=max_cover, adjusted_loglik=adjusted_loglik, n_jobs=n_jobs, start_est=start_est,
-        apply_weights=apply_weights, regul_alpha=regul_alpha, regul_n=regul_n, regul_slice=regul_slice, regul_prior=regul_prior,
+        apply_weights=apply_weights, regul_alpha=regul_alpha if regul else 0, regul_n=regul_n, regul_slice=regul_slice, regul_prior=regul_prior,
         fix_params=fix_params, std=std, optimizer=optimizer, r_transform=None if r_transform == 'none' else r_transform,
         symmetrify=symmetrify, small_dataset_strategy=small_dataset_strategy, small_dataset_n=small_dataset_n,
         stop_slice_n=stop_slice_n)
     if pretty:
         p.stop()
-    update_history(name, 'fit', dist=dist, model=model, left=left, estimate_p=estimate_p, window_size=window_size, 
+    update_history(name, 'fit', dist=dist, left=left, estimate_p=estimate_p, window_size=window_size, 
                    window_behavior=window_behavior, min_slices=min_slices, adjust_line=adjust_line, k_left_bound=k_left_bound,
                    max_count=max_count, max_cover=max_cover, adjusted_loglik=adjusted_loglik, n_jobs=n_jobs, 
-                   regul_alpha=regul_alpha, regul_n=regul_n, regul_slice=regul_slice, regul_prior=regul_prior,
+                   regul=regul, regul_alpha=regul_alpha, regul_n=regul_n, regul_slice=regul_slice, regul_prior=regul_prior,
                    fix_params=fix_params, std=std, optimizer=optimizer, r_transform=r_transform, symmetrify=symmetrify,
                    small_dataset_strategy=small_dataset_strategy, small_dataset_n=small_dataset_n, stop_slice_n=stop_slice_n)
     dt = time() - t0
@@ -440,13 +452,13 @@ def _fit(name: str = Argument(..., help='Project name.'),
 @app.command('test')
 def _test(name: str = Argument(..., help='Project name.'),
           fit: str = Option(None, help='Path to a fit file from a different project. If not provided, fit from the same project is used.'),
-          correction: Correction = Option(Correction.none.value, help='Posterior weight correction method. It effectively helps to choose'
+          correction: Correction = Option(Correction.single.value, help='Posterior weight correction method. It effectively helps to choose'
                                                                       ' a particular component of a distribution for further tests, neglecting '
                                                                       ' an impact of more distant component.'),
           gof_thr: float = Option(None, help='Conservative scoring will be used if goodness-of-fit statistic (RMSEA) exceeds [cyan]gof-thr[/cyan] '
                                                 'for a particular slice.'),
           stop_slice_n_thr: int = Option(-1, help='Conservative scoring will be used if number of samples at a slice is below '
-                                                  '[cyan]dataset-n-thr[/cyan] for a particular slice.'),
+                                                  '[cyan]dataset-n-thr[/cyan].'),
           n_jobs: int = Option(1, help='Number of jobs to be run at parallel, -1 will use all available threads.'),
           pretty: bool = Option(True, help='Use "rich" package to produce eye-candy output.')):
     """
@@ -464,8 +476,8 @@ def _test(name: str = Argument(..., help='Project name.'),
     test(name, correction=correction, gof_tr=gof_thr, fit=fit, dataset_n_thr=stop_slice_n_thr, n_jobs=n_jobs)
     if pretty:
         p.stop()
-    update_history(name, 'test', correction=correction, gof_tr=gof_thr, fit=fit, 
-                   dataset_n_thr=stop_slice_n_thr, n_jobs=n_jobs)
+    update_history(name, 'test', correction=correction, gof_thr=gof_thr, fit=fit, 
+                   stop_slice_n_thr=stop_slice_n_thr, n_jobs=n_jobs)
     dt = time() - t0
     if pretty:
         rprint(f'[green][bold]✔️[/bold] Done![/green]\t time: {dt:.2f} s.')
@@ -475,11 +487,19 @@ def _test(name: str = Argument(..., help='Project name.'),
 @app.command('combine')
 def _combine(name: str = Argument(..., help='Project name.'),
              group: List[Path] = Option(None, '--group', '-g', help='A list of either filenames (vcf or BAM-like tabulars) or folders that contain '
-                                                                      'those filenames or file(s) that contain a list of paths to files.'
+                                                                      'those filenames or file(s) that contain a list of paths to files, or a mask'
+                                                                      ' (masks should start with "[yellow]m:[/yellow]"'
+                                                                      ' prefix, e.g. "m:vcfs/*_M_*.vcf.gz"). '
                                                                       ' SNV p-values from those files shall be combined via logit method.'),
              alpha: float = Option(0.05, help='FWER, family-wise error rate.'),
-             min_cover: int = Option(20, help='If none one of combined p-values is associated with a sample whose ref + alt exceeds'
+             min_cover: int = Option(20, help='If none of combined p-values is associated with a sample whose ref + alt exceeds'
                                               ' [cyan]min_cover[/cyan], the SNV is omitted.'),
+             adaptive_min_cover: bool = Option(False, help='Use adaptive [cyan]min_cover[/cyan] for each BAD. The algorithm has two hyperparameters:'
+                                               '[cyan]adaptive_es[/cyan] and [cyan]adaptive_pval[/cyan]. The minimal coverage where effect-size of'
+                                               'at least [cyan]adative_es[/cyan] is achievable for a p-value of [cyan]adaptive_pval[/cyan].'),
+             adaptive_es: float = Option(1.0, help='Minimal required effect-size for the adaptive coverage algorithm.'),
+             adaptive_pval: float = Option(0.05, help='Minimal required p-value for the adaptive coverage algorithm.'),
+             uniform_weights: bool = Option(True, help='Uniform weighing for effect-sizes.'),
              filter_id: str = Option(None, help='Only SNVs whose IDs agree with this regex pattern are tested (e.g. "rs\w+").'),
              filter_chr: str = Option(None, help='SNVs with chr that does not align with this regex pattern are filtered (e.g. "chr\d+").'),
              subname: str = Option(None, help='You may give a result a subname in case you plan to use multiple groups.'),
@@ -501,8 +521,11 @@ def _combine(name: str = Argument(..., help='Project name.'),
         subname = str(subname)
     else:
         subname = None
-    r = combine(name, group_files=group, alpha=alpha, filter_id=filter_id, filter_chr=filter_chr,
-                subname=subname, min_cnt_sum=min_cover, n_jobs=n_jobs)[subname]['snvs']
+    r, adaptive_coverage = combine(name, group_files=group if group else [r'm:*'], alpha=alpha, filter_id=filter_id, filter_chr=filter_chr,
+                                   subname=subname, min_cnt_sum=min_cover, adaptive_min_cover=adaptive_min_cover,
+                                   adaptive_es=adaptive_es, adaptive_pval=adaptive_pval,
+                                   uniform_weights=uniform_weights, n_jobs=n_jobs)
+    r = r[subname]['snvs']
     if pretty:
         p.stop()
     ref = alt = both = total = 0
@@ -514,16 +537,35 @@ def _combine(name: str = Argument(..., help='Project name.'),
         both += (pv_ref < alpha) & (pv_alt < alpha)
     expected_res = [int(ref), int(alt), int(both)]
     if pretty:
-        rprint('Number of significant SNVs after FDR correction:')
+        if adaptive_min_cover:
+            rprint('Minimal required coverages for each BAD:')
+            table = Table('BAD', 'Ref', 'Alt', 'Min')
+            for bad in sorted(adaptive_coverage['ref']):
+                cov_ref = adaptive_coverage['ref'][bad]
+                cov_alt = adaptive_coverage['alt'][bad]
+                table.add_row(*map(str, (bad, cov_ref, cov_alt, min(cov_ref, cov_alt))))
+            rprint(table)
+        rprint('Number of significantly imbalanced SNVs after FDR correction:')
         table = Table('Ref', 'Alt', 'Both', 'Total significant\n(Percentage of total SNVs)')
         table.add_row(str(ref), str(alt), str(both), f'{total} ({total/len(r) * 100:.2f}%)')
         rprint(table)
+        rprint(f'Total SNVs tested: {len(r)}')
     else:
-        print('Number of significant SNVs after FDR correction:')
+        if adaptive_min_cover:
+            print('Minimal required coverages for each BAD:')
+            table = print('\t'.join(('BAD', 'Ref', 'Alt', 'Min')))
+            
+            for bad in sorted(adaptive_coverage['ref']):
+                cov_ref = adaptive_coverage['ref'][bad]
+                cov_alt = adaptive_coverage['alt'][bad]
+                print('\t'.join(map(str, (bad, cov_ref, cov_alt, min(cov_ref, cov_alt)))))
+        print('Number of significantly imbalanced SNVs after FDR correction:')
         print('\t'.join(('Ref', 'Alt', 'Both', 'Total significant (Percentage of total SNVs)')))
         print('\t'.join((str(ref), str(alt), str(both), f'{total} ({total/len(r) * 100:.2f}%)')))
-    update_history(name, 'combine', group=group, alpha=alpha, min_cover=min_cover, filter_id=filter_id, subname=subname, filter_chr=filter_chr,
-                   n_jobs=n_jobs, expected_result=expected_res)
+        print(f'Total SNVs tested: {len(r)}')
+    update_history(name, 'combine', group=group, alpha=alpha, min_cover=min_cover, adaptive_min_cover=adaptive_min_cover,
+                   adaptive_es=adaptive_es, adaptive_pval=adaptive_pval, filter_id=filter_id, subname=subname, filter_chr=filter_chr,
+                   uniform_weights=uniform_weights, n_jobs=n_jobs, expected_result=expected_res)
     dt = time() - t0
     if pretty:
         rprint(f'[green][bold]✔️[/bold] Done![/green]\t time: {dt:.2f} s.')
@@ -568,7 +610,7 @@ def _difftest(name: str = Argument(..., help='Project name.'),
               n_jobs: int = Option(1, help='Number of jobs to be run at parallel, -1 will use all available threads.'),
               pretty: bool = Option(True, help='Use "rich" package to produce eye-candy output.')):
     """
-    Differential expression tests via likelihood ratio.
+    Differential expression tests via Wald/LR tests.
     """
     t0 = time()
     group_a = str(group_control)
@@ -609,16 +651,22 @@ def _difftest(name: str = Argument(..., help='Project name.'),
     ref = ref.sum()
     alt = alt.sum()
     if pretty:
-        rprint('Number of significantly differentially expressed SNVs after FDR correction:')
-        table = Table('Ref', 'Alt', 'Both', 'Total\nPercentage of total SNVs ')
-        table.add_row(str(ref), str(alt), str(both), f'{total} ({total/len(r) * 100:.2f}%)')
-        rprint(table)
-        rprint('Total SNVs tested:', len(r))
+        if not total:
+            rprint('No SNVs passed filters for differential test.')
+        else:
+            rprint('Number of significantly differentially expressed SNVs after FDR correction:')
+            table = Table('Ref', 'Alt', 'Both', 'Total\nPercentage of total SNVs ')
+            table.add_row(str(ref), str(alt), str(both), f'{total} ({total/len(r) * 100:.2f}%)')
+            rprint(table)
+            rprint('Total SNVs tested:', len(r))
     else:
-        print('Number of significantly differentially expressed SNVs after FDR correction:')
-        print('\t'.join('Ref', 'Alt', 'Both', 'Total/Percentage of total SNVs'))
-        print('\t'.join((str(ref), str(alt), str(both), f'{total} ({total/len(r) * 100:.2f}%)')))
-        rprint('Total SNVs tested:', len(r))
+        if not total:
+            print('No SNVs passed filters for differential test.')
+        else:
+            print('Number of significantly differentially expressed SNVs after FDR correction:')
+            print('\t'.join('Ref', 'Alt', 'Both', 'Total/Percentage of total SNVs'))
+            print('\t'.join((str(ref), str(alt), str(both), f'{total} ({total/len(r) * 100:.2f}%)')))
+            print('Total SNVs tested:', len(r))
     expected_res = [int(ref), int(alt), int(total)]
     update_history(name, 'difftest', group_control=group_a, group_test=group_b, alpha=alpha, min_samples=min_samples, min_cover=min_cover,
                    mode=mode, subname=subname, test_groups=test_groups, max_cover=max_cover, filter_id=filter_id,
@@ -632,6 +680,79 @@ def _difftest(name: str = Argument(..., help='Project name.'),
         print(f'✔️ Done!\t time: {dt:.2f} s.')
     return expected_res
 
+@app.command('anova')
+def _anova(name: str = Argument(..., help='Project name.'),
+           groups: str = Option(str(), help='Files with lists of filenames, folder or a mask (masks should start with "[yellow]m:[/yellow]"'
+                                             'prefix, e.g. "m:vcfs/*_M_*.vcf.gz"), each for a group. Entries must be separated with a semicolon ";" character.'),
+           fit: str = Option(None, help='Path to a fit file from a different project. If not provided, fit from the same project is used.'),
+           param_window: bool = Option(True, help='If disabled, parameters will be taken from a line with respect to the mean window for given'
+                                                     ' reps/samples.'),
+           es_mode: ESMode = Option(ESMode.entropy.value, help='Effect-size calculation method.'),
+           alpha: float = Option(0.05, help='FWER, family-wise error rate.'),
+           min_samples: int = Option(2, help='Minimal number of samples/reps per an SNV to be considered for the analysis.'),
+           min_groups: int = Option(2, help='Minimal number of groups present for an SNV to be considered for the analysis.'),
+           min_cover: int = Option(None, help='Minimal required cover (ref + alt) for an SNV to be considered.'),
+           max_cover: int = Option(None, help='Maximal allowed cover (ref + alt) for an SNV to be considered.'),
+           subname: str = Option(None, help='You may give a result a subname in case you plan to draw multiple comparisons.'),
+           n_jobs: int = Option(1, help='Number of jobs to be run at parallel, -1 will use all available threads.'),
+           pretty: bool = Option(True, help='Use "rich" package to produce eye-candy output.')):
+    """
+    ANOVA test via likelihood ratio.
+    """
+    t0 = time()
+    groups = str(groups)
+    if type(es_mode) is ESMode:
+        es_mode = es_mode.value
+    if pretty:
+        p = Progress(SpinnerColumn(speed=0.5), TextColumn("[progress.description]{task.description}"), transient=True)
+        p.add_task(description='Performing ANOVA tests...', total=None)
+        p.start()
+    else:
+        print('Performing ANOVA tests...')
+    if subname:
+        subname = str(subname)
+    else:
+        subname = None
+    r = anova_test(name, groups=groups.split(';') if groups else None, min_samples=min_samples, min_cover=min_cover,
+                   max_cover=max_cover, alpha=alpha, n_jobs=n_jobs, min_groups=min_groups, es_mode=es_mode,
+                   param_mode='window' if param_window else 'line', fit=fit)[subname]
+    if pretty:
+        p.stop()
+    
+    r = r['tests']
+    ref = r['ref_fdr_pval'] < alpha
+    alt = r['alt_fdr_pval'] < alpha
+    both = (ref & alt).sum()
+    total = (ref | alt).sum()
+    ref = ref.sum()
+    alt = alt.sum()
+    if pretty:
+        if not total:
+            rprint('No SNVs passed filters for ANOVA test.')
+        else:
+            rprint('Number of significantly differentially expressed SNVs after FDR correction:')
+            table = Table('Ref', 'Alt', 'Both', 'Total\nPercentage of total SNVs ')
+            table.add_row(str(ref), str(alt), str(both), f'{total} ({total/len(r) * 100:.2f}%)')
+            rprint(table)
+            rprint('Total SNVs tested:', len(r))
+    else:
+        if not total:
+            print('No SNVs passed filters for differential test.')
+        else:
+            print('Number of significantly differentially expressed SNVs after FDR correction:')
+            print('\t'.join('Ref', 'Alt', 'Both', 'Total/Percentage of total SNVs'))
+            print('\t'.join((str(ref), str(alt), str(both), f'{total} ({total/len(r) * 100:.2f}%)')))
+            print('Total SNVs tested:', len(r))
+    expected_res = [int(ref), int(alt), int(total)]
+    update_history(name, 'anova', groups=groups, min_samples=min_samples, min_cover=min_cover,
+                          max_cover=max_cover, alpha=alpha, n_jobs=n_jobs, min_groups=min_groups,
+                          param_window=param_window, es_mode=es_mode, fit=fit)
+    dt = time() - t0
+    if pretty:
+        rprint(f'[green][bold]✔️[/bold] Done![/green]\t time: {dt:.2f} s.')
+    else:
+        print(f'✔️ Done!\t time: {dt:.2f} s.')
+    return expected_res
 
 
 @app_export.command('all', help='Export everything.')
@@ -769,7 +890,7 @@ def _difftests(name: str = Argument(..., help='Project name.'), out: Path = Argu
 def _demo(export_path: Path = Option(str(), help='Path where the demo data will be extracted.'),
           pretty: bool = Option(True, help='Use "rich" package to produce eye-candy output.')):
     '''
-    Extract demonstration data from ChIP-seq experiment.
+    Extract demonstration data from DNase-seq experiment for the cell line K562.
     '''
     export_path = str(export_path)
     export.export_demo(export_path)
@@ -808,9 +929,11 @@ def _plot_all(name: str = Argument(..., help='Project name.'), out: Path = Argum
 
 @app.command('test_binom')
 def _test_binom(name: str = Argument(..., help='Project name.'),
-          w: str = Option('1', help='Right mode weight. It can be a formulae that uses both "n" and "bad".'),
-          n_jobs: int = Option(1, help='Number of jobs to be run at parallel, -1 will use all available threads.'),
-          pretty: bool = Option(True, help='Use "rich" package to produce eye-candy output.')):
+                beta: bool = Option(False, help='Use beta-binomial model instead of binomial. In that case, concentration paremeter will be'
+                                                ' estimated from the data for each BAD.'),
+                w: str = Option(str(), help='Left mode weight. If None, then p=1/2, w=1 will be used everywhere.'),
+                n_jobs: int = Option(1, help='Number of jobs to be run at parallel, -1 will use all available threads.'),
+                pretty: bool = Option(True, help='Use "rich" package to produce eye-candy output.')):
     """
     Calculate p-values with a non-bias adjusted left truncated binomial model.
     """
@@ -821,10 +944,22 @@ def _test_binom(name: str = Argument(..., help='Project name.'),
         p.start()
     else:
         print('Computing p-values and effect sizes...')
-    binom_test(name, w=w, n_jobs=n_jobs)
+    _, params = binom_test(name, w=None if not w else float(w), beta=beta, n_jobs=n_jobs)
     if pretty:
+        if beta:
+            table = Table('BAD', 'ref', 'alt')
+            for bad in sorted(params):
+                d = params[bad]
+                ref = '{:.3f}'.format(d['ref'])
+                alt = '{:.3f}'.format(d['alt'])
+                table.add_row('{:.2f}'.format(bad), ref, alt)
+            rprint('Estimated concentration parameters:')
+            rprint(table)
         p.stop()
-    update_history(name, 'binom_test', w=w, n_jobs=n_jobs)
+    else:
+        print('Estimated concentration parameters:')
+        print(params)
+    update_history(name, 'binom_test', w=w, beta=beta, n_jobs=n_jobs)
     dt = time() - t0
     if pretty:
         rprint(f'[green][bold]✔️[/bold] Done![/green]\t time: {dt:.2f} s.')
