@@ -6,7 +6,7 @@ from betanegbinfit import ModelMixture
 import matplotlib.pyplot as plt
 import numpy as np
 from matplotlib.ticker import FormatStrFormatter
-from .utils import openers, get_init_file, get_model_creator, dictify_params
+from .utils import openers, get_init_file, get_model_creator, dictify_params, scorefiles_qc
 from betanegbinfit.utils import get_params_at_slice
 from scipy.interpolate import UnivariateSpline
 import dill
@@ -42,9 +42,12 @@ def plot_heatmap(counts: np.ndarray, max_count: int, slices=None, shift=10, cmap
 
     plt.imshow(hm, cmap=cmap, vmin=0, vmax=max_order)
     if slices:
-        a, b = slices
-        plt.vlines(a, 0, max_count, colors=_alt, linestyles='dashed', linewidth=3)
-        plt.hlines(b, 0, max_count, colors=_ref, linestyles='dashed', linewidth=3)
+        try:
+            a, b = slices
+            plt.vlines(a, 0, max_count, colors=_alt, linestyles='dashed', linewidth=3)
+            plt.hlines(b, 0, max_count, colors=_ref, linestyles='dashed', linewidth=3)
+        except TypeError:
+            plt.plot([0, slices], [slices, 0], color='k', linestyle='dashed', linewidth=3)
     plt.xlim(0, max_count)
     plt.ylim(0, max_count)
     plt.xlabel('Reference allele read count')
@@ -62,10 +65,12 @@ def get_pdf_computer(m: ModelMixture, params: dict):
     return lambda x, y: np.exp(m.logprob(m.dict_to_vec(get_params_at_slice(params, y)), x))
 
     
-def plot_histogram(counts: np.ndarray, max_count: int, slc: int, pdf_computer, s=0, c='r'):
-    
+def plot_histogram(counts: np.ndarray, max_count: int, slc: int, pdf_computer, s=0, c='r', slc_sum=False):
     counts = counts[counts[:, s] < max_count, :]
-    counts = counts[counts[:, 1 - s] == slc][:, [s, 2]]
+    if slc_sum:
+        counts = counts[counts[:, 1 - s] + counts[:, s] == slc][:, [s, 2]]
+    else:
+        counts = counts[counts[:, 1 - s] == slc][:, [s, 2]]
     plt.bar(counts[:, 0], counts[:, 1] / counts[:, 1].sum(), width=1, color=_count)
     x = np.arange(0, max_count)
     y = pdf_computer(x, slc)
@@ -146,9 +151,58 @@ def plot_stat(stats_ref: dict, stats_alt: dict, max_count: int, stat: str, figsi
     plt.xlabel('Read count for the fixed allele')
     plt.ylabel(r'$log_{10}$' + stat if log else stat)
 
+def plot_scorefiles_qc(covers: dict, biases: dict, scorefiles: list, bad=None, rule=3.0,
+                       figsize=(6, 6), dpi=200,):
+    common_prefix = 0
+    common_postfix = 0
+    for i in range(len(min(scorefiles, key=len))):
+        c = scorefiles[0][i]
+        common = True
+        for f in scorefiles[1:]:
+            if c != f[i]:
+                common = False
+                break
+        if not common:
+            break
+        common_prefix += 1
+    for i in range(1, len(min(scorefiles, key=len))):
+        c = scorefiles[0][-i]
+        common = True
+        for f in scorefiles[1:]:
+            if c != f[-i]:
+                common = False
+                break
+        if not common:
+            break
+        common_postfix -= 1
+    if not common_postfix:
+        common_postfix = None
+    scorefiles = [f[common_prefix:common_postfix] for f in scorefiles]
+    labels = list()
+    x = list()
+    y = list()
+    labels = list()
+    for ind in sorted(covers):
+        cover = covers[ind][bad]
+        if cover:
+            x.append(cover)
+            y.append(biases[ind][bad])
+            labels.append(scorefiles[ind])
+    x = np.log10(x)
+    y = np.array(y)
+    labels = np.array(labels)
+    inds = np.abs(y - np.mean(y)) > rule * np.std(y)
+    plt.figure(figsize=(6, 6), dpi=200)
+    plt.scatter(x, y, s=15, c=-np.array(y))
+    for xc, yc, label in zip(x[inds], y[inds], labels[inds]):
+        plt.annotate(label, (xc, yc), fontsize=8)
+    plt.xlabel(r'$log_{10}(\mathrm{coverage})$')
+    plt.ylabel(r'Fraction of SNVs when $ref > alt$')
+    plt.grid(True)
+
 def plot_params(params_ref: dict, params_alt: dict, max_count: int, param: str,
-                figsize=(6, 6), dpi=200, inv=False, diag=False, name=None, spline=False,
-                std=True):
+                figsize=(6, 6), dpi=200, inv=False,  diag=False, name=None, spline=False,
+                hor_expected=None, std=True):
     x = np.arange(0, max_count)
     if std:
         try:
@@ -160,9 +214,26 @@ def plot_params(params_ref: dict, params_alt: dict, max_count: int, param: str,
             std = False
     params_ref = dictify_params(params_ref)
     params_alt = dictify_params(params_alt)
-    # min_cnt = min(int(k[2:]) for k in params_ref if k.startswith('mu') and k[2].isdigit())    
-    pref = np.array([get_params_at_slice(params_ref, i, clip_at_max_slice=False, nan_min=True).get(param, np.nan) for i in x])
-    palt = np.array([get_params_at_slice(params_alt, i, clip_at_max_slice=False, nan_min=True).get(param, np.nan) for i in x])
+    if type(param) is not str:
+        if len(param) == 2:
+            pref1 = np.array([get_params_at_slice(params_ref, i, clip_at_max_slice=False, nan_min=True).get(param[0], np.nan) for i in x])
+            palt1 = np.array([get_params_at_slice(params_alt, i, clip_at_max_slice=False, nan_min=True).get(param[0], np.nan) for i in x])
+            pref2 = np.array([get_params_at_slice(params_ref, i, clip_at_max_slice=False, nan_min=True).get(param[1], np.nan) for i in x])
+            palt2 = np.array([get_params_at_slice(params_alt, i, clip_at_max_slice=False, nan_min=True).get(param[1], np.nan) for i in x])
+            pref = pref1 + pref2 
+            palt = palt1 + palt2
+        else:
+            pref1 = np.array([get_params_at_slice(params_ref, i, clip_at_max_slice=False, nan_min=True).get(param[0], np.nan) for i in x])
+            palt1 = np.array([get_params_at_slice(params_alt, i, clip_at_max_slice=False, nan_min=True).get(param[0], np.nan) for i in x])
+            pref2 = np.array([get_params_at_slice(params_ref, i, clip_at_max_slice=False, nan_min=True).get(param[1], np.nan) for i in x])
+            palt2 = np.array([get_params_at_slice(params_alt, i, clip_at_max_slice=False, nan_min=True).get(param[1], np.nan) for i in x])
+            pref3 = np.array([get_params_at_slice(params_ref, i, clip_at_max_slice=False, nan_min=True).get(param[2], np.nan) for i in x])
+            palt3 = np.array([get_params_at_slice(params_alt, i, clip_at_max_slice=False, nan_min=True).get(param[2], np.nan) for i in x])
+            pref = pref3 / (pref1 + pref2)
+            palt = palt3 / (palt1 + palt2)
+    else:
+        pref = np.array([get_params_at_slice(params_ref, i, clip_at_max_slice=False, nan_min=True).get(param, np.nan) for i in x])
+        palt = np.array([get_params_at_slice(params_alt, i, clip_at_max_slice=False, nan_min=True).get(param, np.nan) for i in x])
     if inv:
         pref = 1 / pref
         palt = 1 / palt
@@ -187,6 +258,8 @@ def plot_params(params_ref: dict, params_alt: dict, max_count: int, param: str,
         plt.fill_between(x, palt - salt, palt + salt, alpha=0.2, color=_alt)
     if diag:
         plt.axline((1, 1), slope=1, linestyle='dashed', color='k')
+    if hor_expected is not None:
+        plt.axhline(hor_expected, linestyle='dashed', color='k')
     plt.grid(True)
     plt.legend(['ref', 'alt'])
     plt.xlabel('Read count for the fixed allele')
@@ -199,11 +272,23 @@ def visualize(name: str, output: str, what: str, fmt='png', slices=(5, 10, 15, 2
     compressor = filename.split('.')[-1]
     open = openers[compressor]
     with open(filename, 'rb') as f:
-        counts = dill.load(f)['counts']
+        counts = dill.load(f)
+        covers, biases = scorefiles_qc(counts)
+        scorefiles = counts['scorefiles']
+        counts = counts['counts']
     filename = f'{name}.fit.{compressor}'
     with open(filename, 'rb') as f:
         fits = dill.load(f)
-    bads = [fbad] if fbad else sorted(counts) 
+    bads = [fbad] if fbad else sorted(counts)
+    if what == 'all' and fbad is None:
+        os.makedirs(output, exist_ok=True)
+        filename = os.path.join(output, f'scorefiles_qc.{fmt}')
+        plot_scorefiles_qc(covers, biases, scorefiles)
+        if show_bad:
+            plt.title('All BADs')
+        plt.tight_layout()
+        plt.savefig(filename, bbox_inches='tight')
+        
     for bad in bads:
         if not fbad:
             subfolder = os.path.join(output, f'BAD{bad:.2f}')
@@ -215,6 +300,12 @@ def visualize(name: str, output: str, what: str, fmt='png', slices=(5, 10, 15, 2
         m = get_model_creator(**inst_params)()
         if what == 'all':
             os.makedirs(subfolder, exist_ok=True)
+            filename = os.path.join(subfolder, f'scorefiles_qc.{fmt}')
+            plot_scorefiles_qc(covers, biases, scorefiles, bad=bad, dpi=dpi)
+            plt.tight_layout()
+            if show_bad:
+                plt.title(f'BAD = {bad:.2f}')
+            plt.savefig(filename, bbox_inches='tight')
             filename = os.path.join(subfolder, f'gof.{fmt}')
             plot_gof(fits['ref'][bad]['stats'], fits['alt'][bad]['stats'], max_count, dpi=dpi)
             if show_bad:
@@ -250,6 +341,48 @@ def visualize(name: str, output: str, what: str, fmt='png', slices=(5, 10, 15, 2
                     plt.savefig(filename, bbox_inches='tight')
                 except KeyError:
                     pass
+            filename = os.path.join(subfolder, f'p1.{fmt}')
+            try:
+                plot_params(fits['ref'][bad]['params'], fits['alt'][bad]['params'], max_count, 'p1', name='$p_1$', dpi=dpi,
+                            hor_expected=bad/(bad + 1)
+                            )
+                if show_bad:
+                    plt.title(f'BAD = {bad:.2f}')
+                plt.tight_layout()
+                plt.savefig(filename, bbox_inches='tight')
+            except KeyError:
+                pass
+            if bad > 1:
+                filename = os.path.join(subfolder, f'p2.{fmt}')
+                try:
+                    plot_params(fits['ref'][bad]['params'], fits['alt'][bad]['params'], max_count, 'p2', name='$p_2$', dpi=dpi,
+                                hor_expected=1/(bad + 1))
+                    if show_bad:
+                        plt.title(f'BAD = {bad:.2f}')
+                    plt.tight_layout()
+                    plt.savefig(filename, bbox_inches='tight')
+                except KeyError:
+                    pass
+                filename = os.path.join(subfolder, f'p0.{fmt}')
+                try:
+                    plot_params(fits['ref'][bad]['params'], fits['alt'][bad]['params'], max_count, ('p1', 'p2'), name='$p_0$', dpi=dpi,
+                                hor_expected=1.0, std=False)
+                    if show_bad:
+                        plt.title(f'BAD = {bad:.2f}')
+                    plt.tight_layout()
+                    plt.savefig(filename, bbox_inches='tight')
+                except KeyError:
+                    pass
+                filename = os.path.join(subfolder, f'p.{fmt}')
+                try:
+                    plot_params(fits['ref'][bad]['params'], fits['alt'][bad]['params'], max_count, ('p1', 'p2', 'p1'), name='$p$', dpi=dpi,
+                                hor_expected=bad/(bad + 1), std=False)
+                    if show_bad:
+                        plt.title(f'BAD = {bad:.2f}')
+                    plt.tight_layout()
+                    plt.savefig(filename, bbox_inches='tight')
+                except KeyError:
+                    pass
             filename = os.path.join(subfolder, f'n.{fmt}')
             plot_stat(fits['ref'][bad]['stats'], fits['alt'][bad]['stats'], max_count, 'n', dpi=dpi, log=True)
             if show_bad:
